
var FILE = require("modules/file"),
	JSLINT = require("jslint/jslint").JSLINT,
	JSLINT_D = require("jslint-d/jslint").JSLINT;

// Also look into: https://github.com/jshint/jshint

exports.main = function(env)
{
	// Validate a JS script as loosely as possible
	runner("script-all", {
		'white':true,		// Sloppy whitespace (enabled in "good parts")
		'undef':true,		// if variables can be declared out of order
		'devel':true,		// logging should be allowed (console, alert, etc.)
		'eqeq':true,		// == should be allowed
		'forin':true,		// for in statements need not filter
		'sloppy':true,		// the 'use strict'; pragma is optional
		'vars':true			// if multiple var statements per function should be allowed
	});

	// Validate a CommonJS module as loosly as possible
	runner("module-loose", {
		'white':true,		// Sloppy whitespace (enabled in "good parts")
		'undef':true,		// Disallow undefined variables (enabled in "good parts")
		'devel':true,		// logging should be allowed (console, alert, etc.)
		'eqeq':true,		// == should be allowed
		'forin':true,		// for in statements need not filter
		'sloppy':true,		// the 'use strict'; pragma is optional
		'vars':true,		// if multiple var statements per function should be allowed
		'commonjs':true		// no effect since globals are allowed when 'safe' not set
	});

	// Validate a CommonJS module strictly but not paranoid
	runner("module-strict");
}

function runner(moduleId, policy)
{
	module.print("\0magenta(Test module: " + moduleId + "\0)\n");

	try
	{
	    module.print("this before module load:\n");
	    for (var name in this)
	    {
   		 	module.print("  " + name + ": " + (typeof this[name]) + "\n");
	    }
		
	    var source = FILE.read(FILE.dirname(module.id) + "/" + moduleId + ".js");

	    var globals = checkSource(JSLINT, source, policy);

	    module.print("Globals (via jslint):\n");
	    if (globals.length > 0)
	    {
		    for (var i=0; i<globals.length ; i++)
		    {
	   		 	module.print("  " + globals[i] + "\n");
		    }
	    }

	    // CommonJS module globals
	    var fRequire = function()
		    {
		    },
		    fExports = {
		    },
		    fModule = {
		    	print: module.print
		    };
	    
		module.print("Load module: " + moduleId + "\n");
	    factory(source, fRequire, fExports, fModule);
	    
	    module.print("Exports:\n");
	    for (var name in fExports)
	    {
   		 	module.print("  " + name + ": " + (typeof fExports[name]) + "\n");
	    }
	    
	    module.print("this after module load:\n");
	    for (var name in this)
	    {
   		 	module.print("  " + name + ": " + (typeof this[name]) + "\n");
	    }
	}
	catch(e)
	{
	    console.error("Module loader error: " + e, e);
	}
}

function factory(source, require, exports, module)
{
    try
    {
    	var globals = {};
    	
	    module.print("this before source run:\n");
	    for (var name in this)
	    {
	        globals[name] = this[name];
   		 	module.print("  " + name + ": " + (typeof globals[name]) + "\n");
   		 	delete this[name];
	    }

	    eval(source);
	}
	catch (e)
	{
	    console.error("Module script error: " + e, e);
	}
	finally
	{
		module.print("New globals:\n");
	    for (var name in this)
	    {
	        if (typeof globals[name] === "undefined")
	        {
	     		module.print("  " + name + ": " + (typeof this[name]) + "\n");
	     		delete this[name];
	        }
	    }
	    for (var name in globals)
	    {
	    	this[name] = globals[name];
	    }
	}
}

function checkSource(parser, source, policy)
{
	// Detect commonjs 'slashstar comment' options
	var stcre = /\n\/\*commonjs\s*\n((\n|.)*?)\n\*\//g,
		pre = /\s*([^:]*):\s*(\S*)(,|\s|\n)?/g,
		m1, m2,
		commonjsOptions = {};
	while (m1 = stcre.exec(source))
	{
		while (m2 = pre.exec(m1[1]))
		{
			commonjsOptions[m2[1]] = m2[2];
		}
	}
	
    module.print("commonjs options:\n");
    for (var name in commonjsOptions)
    {
	 	module.print("  " + name + ": " + commonjsOptions[name] + "\n");
    }
	
	policy = policy || false;

	if (typeof commonjsOptions["jslint.policy"] !== "undefined")
	{
		if (commonjsOptions["jslint.policy"] === "github.com/pinf/loader-js/-meta/strawman/module-source-format/commonjs-safe/0.1")
		{
			module.print("... validating against policy: " + commonjsOptions["jslint.policy"] + "\n");

			policy = {
				'white':true,		// Sloppy whitespace (enabled in "good parts")
				'undef':true,		// if variables can be declared out of order
				'devel':true,		// logging should be allowed (console, alert, etc.)
				'eqeq':true,		// == should be allowed
				'forin':true,		// for in statements need not filter
				'sloppy':true,		// the 'use strict'; pragma is optional
				'vars':true,		// if multiple var statements per function should be allowed
				'commonjs':true,
				'safe':true
			};
		}
	}

	if (!policy)
		return [];
	
	var result = parser(source, policy);

	if (!result)
		module.print("... done with errors\n");

	if (!result)
		console.log("JSLINT.errors", parser.errors);

	return parser.data().globals || [];
}
